/*
    extendable array library
            12/Oct/2004   by Takeaki Uno  e-mail:uno@nii.jp, 
    homepage:   http://research.nii.ac.jp/~uno/index.html  */
/* This program is available for only academic use, basically.
   Anyone can modify this program, but he/she has to write down 
    the change of the modification on the top of the source code.
   Neither contact nor appointment to Takeaki Uno is needed.
   If one wants to re-distribute this code, do not forget to 
    refer the newest code, and show the link to homepage of 
    Takeaki Uno, to notify the news about the codes for the users.
   For the commercial use, please make a contact to Takeaki Uno. */

#ifndef _ary_c_
#define _ary_c_

#include"ary.h"

/* print routine */
void ARY_print ( ARY *A ){
  printf ("ARY %p, num=%d, end=%d, unit=%d, h=%p\n",
           A, A->num, A->end, A->unit, A->h );
}

  /* expand the array to num. num is accessible. */
void ARY_exp ( ARY *A, int num ){
  if ( num >= A->end ){
    A->end = MAX(A->end*2, num+1);
    realloc2 ( A->h, char, A->unit*A->end, "ARY_exp: A->h" );
    A->num = num+1;
  } else if ( A->num <= num ) A->num = num+1;
}

  /* increment num */
void ARY_inc ( ARY *A ){
  ARY_exp ( A, A->num );
}


  /* expand the array, and fill the new part by c */
void ARY_exp_const ( ARY *A, int num, int c ){
  int *x, *x_end, end = A->end;
  ARY_exp ( A, num );
  for (x=((int *)(A->h))+end,x_end=((int *)(A->h))+A->end ; x<x_end ;x++) *x=c;
}

  /* set num-th unitu to integer c */
void ARY_set ( ARY *A, int num, int c ){
  ARY_exp ( A, num );
  ((int *)(A->h))[num] = c;
}

  /* increment num, and set num-th unit to c */
void ARY_set_inc ( ARY *A, int c ){
  ARY_set ( A, A->num, c );
}

  /* set num-th unit to c. if it is expanded, set the units in the gap to c2 */
void ARY_set_const ( ARY *A, int num, int c, int c2 ){
  ARY_exp_const ( A, num, c2 );
  ((int *)(A->h))[num] = c;
}

  /* expand ARY to num, and set the units in the gap to *p */
void ARY_exp_const_ ( ARY *A, int num, void *p ){
  void *x, *x_end;
  int end = A->end;
  ARY_exp ( A, num );
  for (x=A->h+end*A->unit,x_end=A->h+A->end*A->unit ; x<x_end ; x+=A->unit )
      memcpy (x, p, A->unit );
}

  /* set num-th unit to *p */
void ARY_set_ ( ARY *A, int num, void *p ){
  ARY_exp ( A, num );
  memcpy ( A->h + num*A->unit, p, A->unit );
}

  /* set A->num-th unit to *p, and increment num */
void ARY_set_inc_ ( ARY *A, void *p ){
  ARY_set_ ( A, A->num, p );
}

  /* set num-th unit to *p. if expanded, set units in the gap to *p2 */
void ARY_set_const_ ( ARY *A, int num, void *p, void *p2 ){
  ARY_exp_const_ ( A, num, p2 );
  memcpy ( A->h + num*A->unit, p, A->unit );
}


  /* initialization */
void ARY_init ( ARY *A, int unit ){
  A->num = 0;
  A->end = 0;
  A->unit = unit;
  A->h = NULL;
  A->dellist = -1;
}

  /* termination */
void ARY_end ( ARY *A ){
  free2 ( A->h );
}

  /* delete one cell and insert the deletion list */
void ARY_del ( ARY *A, int num ){
  *ARY_CELL(int, *A, num) = A->dellist;
  A->dellist = -num-2;
}

  /* delete one cell and move the last unit here */
void ARY_rm ( ARY *A, int num ){
  if ( num<0 || num>=A->num ){
    printf ("ARY_rm: num is out of range %d\n",num);
    exit(1);
  }
  A->num--;
  memcpy ( ((char *)(A->h)) + num*A->unit,
       ((char *)(A->h)) + A->num*A->unit, A->unit );
}

  /* return a new cell, take from the deleted list, or increment num if empty */
int ARY_new ( ARY *A ){
  int num;
  if ( A->dellist != -1 ){
    num = -A->dellist-2;
    A->dellist = *ARY_CELL (int, *A, num);
  } else {
    num = A->num;
    ARY_inc ( A );
  }
  return ( num );
}

#endif

