/* HornSat: Satisfiablity Checking for horn formula functions  */
/* if the function is satisfiable, output the assignment of minimum truth */
/* 22/Jan/2005 Takeaki Uno,  e-mail:uno@nii.jp, 
    homepage:   http://research.nii.ac.jp/~uno/index.html  */
/* This program is available for only academic use, basically.
   Anyone can modify this program, but he/she has to write down 
    the change of the modification on the top of the source code.
   Neither contact nor appointment to Takeaki Uno is needed.
   If one wants to re-distribute this code, do not forget to 
    refer the newest code, and show the link to homepage of 
    Takeaki Uno, to notify the news about codes for the users.
   For the commercial use, please make a contact to Takeaki Uno. */

#ifndef _hornsat_c_
#define _hornsat_c_

#include"sgraph.c"
#include"queue.c"


QUEUE HORNSAT_unsat;
SGRAPH HORNSAT_func;
int *HORNSAT_assign;
QUEUE HORNSAT_true;
int HORNSAT_printflag = 1;
/*  print_flag 
    &1: output, no output if 0
    &2: verbose output
*/

/* initialization */
void HORNSAT_init (){
  int i;
  QUEUE_init ( &HORNSAT_unsat, HORNSAT_func.node1_num );
  HORNSAT_assign = intarray_malloc_const ( HORNSAT_func.node_end, 0 );
  for ( i=0 ; i<HORNSAT_func.node1_num ; i++ ){
    HORNSAT_assign[i] = SGRAPH_DEG(HORNSAT_func,i);
    if ( SGRAPH_DEG(HORNSAT_func,i) == 1 )
        QUEUE_ins_ ( &HORNSAT_unsat, QUEUE_HEAD(HORNSAT_func.edge[i]) );
  }
  QUEUE_init ( &HORNSAT_true, HORNSAT_func.node_end - HORNSAT_func.node1_num);
}

/* main iteration routine */
int HORNSAT_iter (){
  int i, c, x, v;
  while (1){
    if ( QUEUE_LENGTH( HORNSAT_unsat ) == 0 ) return (1);
    x = QUEUE_ext_ ( &HORNSAT_unsat );
    if ( HORNSAT_assign[x] == 1 ) continue;

    QUEUE_FE_LOOP_ (HORNSAT_func.edge[x], i, c){
      v = HORNSAT_func.edge[c].q[0];
      if ( v == x ) continue;
      HORNSAT_assign[c]--;
      if (HORNSAT_assign[c]==1 ) QUEUE_ins_ ( &HORNSAT_unsat, v);
    }
    HORNSAT_assign[x] = 1;
    QUEUE_ins_ ( &HORNSAT_true, x );
  }
}

/* output routine */
void HORNSAT_output (){
  int i;
  if ( HORNSAT_printflag & 2 ) printf ("true/all = %4f\n", ((double)HORNSAT_true.t)/(HORNSAT_func.node_end-HORNSAT_func.node1_num));
  if ( (HORNSAT_printflag &1)== 0 ) return;
  for ( i=HORNSAT_func.node1_num ; i<HORNSAT_func.node_num ; i++ ){
    if ( HORNSAT_assign[i]==0 ) printf("%d false\n", i-HORNSAT_func.node1_num);
    else printf ("%d true\n", i-HORNSAT_func.node1_num);
  }
}

/* termination */
void HORNSAT_end (){
  SGRAPH_end ( &HORNSAT_func );
  QUEUE_end ( &HORNSAT_unsat );
  QUEUE_end ( &HORNSAT_true );
  free2 ( HORNSAT_assign );
}

    /*********************************/
    /***   main routine            ***/
    /*********************************/
#define HORNSAT_RAND 1000000
int main ( int argc, char *argv[] ){
  if ( argc <2 ) goto ERROR;
  if ( argc == 3 && strchr( argv[2], 'q') ) HORNSAT_printflag = 0;
  else HORNSAT_printflag = 1;
  if ( argc == 3 && strchr( argv[2], 'v') ) HORNSAT_printflag |= 2;
  SGRAPH_load (&HORNSAT_func, argv[1], 1, 0, 0, 0, 1);

  if ( HORNSAT_printflag & 2 ) printf ("#clouse=%d, #variables=%d, function size=%d\n", HORNSAT_func.node1_num, HORNSAT_func.node_num-HORNSAT_func.node1_num, HORNSAT_func.edge_end );

  HORNSAT_init ();
  HORNSAT_iter ();
  HORNSAT_output ();
  HORNSAT_end ();
  exit(0);

ERROR:;
  printf ("hornsat: [options] filename\n");
  printf ("Check satisfiablity of horn formula functions in filename\n");
  printf ("If the function is satisfiable, output the assignment of minimum truth\n");
  printf ("[options]\n");
  printf ("q: no output, v:verbose output\n"); 
  exit (1);
}





#endif


