/* HornSat: Satisfiablity Checking for horn formula functions  */
/* if the function is satisfiable, output the assignment of minimum truth */
/* 22/Jan/2005 Takeaki Uno,   e-mail:uno@nii.jp, 
    homepage:   http://research.nii.ac.jp/~uno/index.html  */
/* This program is available for only academic use, basically.
   Anyone can modify this program, but he/she has to write down 
    the change of the modification on the top of the source code.
   Neither contact nor appointment to Takeaki Uno is needed.
   If one wants to re-distribute this code, do not forget to 
    refer the newest code, and show the link to homepage of 
    Takeaki Uno, to notify the news about codes for the users.
   For the commercial use, please make a contact to Takeaki Uno. */

/*  
type QUEUE ::: a structure representing a queue.
 a structure "QUEUE Q" representing a queue has 
  Q.s     starting position of the queue
  Q.t     ending position of the queue
  Q.q     array storing the elements inserted in the queue
  Q.end   the size of the array Q.q

In this implementation, we use them as a simple array, and 
 always set Q.s = 0 for any variable of QUEUE type

QUEUE HORNSAT_unsat_clause ::: the queue of clauses to be evaluated.


SGRAPH HORNSAT_func;
int *HORNSAT_assign;
QUEUE HORNSAT_true;
int HORNSAT_printflag = 1;



*/


#ifndef _hornsat_c_
#define _hornsat_c_

#include"sgraph.c"
#include"queue.c"
#include"ary.c"

SGRAPH HORNSAT_func;
ARY HORNSAT_assign;
ARY HORNSAT_falsenum;
ARY HORNSAT_true;
ARY HORNSAT_clause;
ARY HORNSAT_head;
ARY HORNSAT_tuple;
int HORNSAT_printflag = 1;
ARY HORNSAT_ARYdef;
int HORNSAT_true_s;

void HORNSAT_init (){
  int i;
  ARY_init ( &HORNSAT_true, sizeof(int) );
  ARY_init ( &HORNSAT_tuple, sizeof(ARY) );
  ARY_init ( &HORNSAT_assign, sizeof(int) );
  ARY_init ( &HORNSAT_clause, sizeof(ARY) );
  ARY_init ( &HORNSAT_head, sizeof(int) );
  ARY_init ( &HORNSAT_falsenum, sizeof(int) );
  ARY_init ( &HORNSAT_ARYdef, sizeof(int) );
  HORNSAT_true_s = 0;
}

void HORNSAT_end (){
  ARY_end ( &HORNSAT_true );
  ARY_end ( &HORNSAT_tuple );
  ARY_end ( &HORNSAT_assign );
  ARY_end ( &HORNSAT_clause );
  ARY_end ( &HORNSAT_head );
  ARY_end ( &HORNSAT_falsenum );
}

void HORNSAT_iter (){
  int i, *cl, *cl_end, x, head;
  ARY *A = (ARY *)(HORNSAT_tuple.h);
  while ( HORNSAT_true_s < HORNSAT_true.num ){
    x = ((int *)(HORNSAT_true.h))[HORNSAT_true_s];
    ((int *)(HORNSAT_assign.h))[x] = 1;
    cl = (int *)(A[x].h);
    for ( cl_end = cl+A[x].num ; cl<cl_end ; cl++ ){
      if ( ((int *)(HORNSAT_falsenum.h))[*cl]-- == 1 ){
        head = ((int *)(HORNSAT_head.h))[*cl];
        if ( ((int *)(HORNSAT_assign.h))[head]==0 )
            ARY_set_inc ( &HORNSAT_true, head );
      }
    }
    HORNSAT_true_s++;
  }
}

/* add a horn clause */
void HORNSAT_add_clause ( int head, QUEUE *C ){
  int i, e, id = HORNSAT_clause.num, m=head, flag=0, *x;
  ARY *A, *VA;
  ARY_set_ ( &HORNSAT_clause, id, &HORNSAT_ARYdef );
  ARY_set ( &HORNSAT_falsenum, id, 0 );
  ARY_set ( &HORNSAT_head, id, head );
  A = &(((ARY *)(HORNSAT_clause.h))[id]);
  QUEUE_FE_LOOP_ ( *C, i, e ){
    ARY_set_inc ( A, e );
    if ( e>m ) m=e;
  }
  ARY_exp_const ( &HORNSAT_assign, m, 0 );
  ARY_exp_const_ ( &HORNSAT_tuple, m, &HORNSAT_ARYdef );
  VA = (ARY *)(HORNSAT_tuple.h);
  QUEUE_FE_LOOP_ ( *C, i, e ){
    ARY_set_inc ( &VA[e], id );
    if ( ((int *)(HORNSAT_assign.h))[e] == 0 ) 
        ((int *)(HORNSAT_falsenum.h))[id]++;
  }
  if ( ((int *)(HORNSAT_falsenum.h))[id] == 0
        && ((int *)(HORNSAT_assign.h))[head] == 0 ){
    ARY_set_inc ( &HORNSAT_true, head );
    HORNSAT_iter ();
  }
}

/* output routine */
void HORNSAT_output (){
  int i;
  if ( HORNSAT_printflag & 2 ) printf ("true/all = %4f\n", ((double)HORNSAT_true.num)/HORNSAT_tuple.num);
  if ( (HORNSAT_printflag &1)== 0 ) return;
  for ( i=0 ; i<HORNSAT_assign.num ; i++ ){
    if ( ((int *)(HORNSAT_assign.h))[i]==0 )
        printf("%d false\n", i );
    else printf ("%d true\n", i );
  }
}

/* load data from a file */
void HORNSAT_load ( char *filename ){
  int i, head;
  SGRAPH_load (&HORNSAT_func, filename, 1, 0, 0, 0, 1);
  if ( HORNSAT_printflag & 2 ) printf ("#clouse=%d, #variables=%d, function size=%d\n", HORNSAT_func.node1_num, HORNSAT_func.node_num-HORNSAT_func.node1_num, HORNSAT_func.edge_end );
  for ( i=0 ; i< HORNSAT_func.node1_num ; i++ ){
    if ( QUEUE_LENGTH(HORNSAT_func.edge[i]) == 0 ) continue;

//    head = QUEUE_ext_tail_ (&HORNSAT_func.edge[i]);
    QUEUE_F_LOOP_ ( HORNSAT_func.edge[i], head )
        HORNSAT_func.edge[i].q[head] -= HORNSAT_func.node1_num;
    head = HORNSAT_func.edge[i].q[0];
    HORNSAT_func.edge[i].q[0]=HORNSAT_func.edge[i].q[--HORNSAT_func.edge[i].t];

    HORNSAT_add_clause ( head, &HORNSAT_func.edge[i] );
  }
  SGRAPH_end ( &HORNSAT_func );
}



    /*********************************/
    /***   main routine            ***/
    /*********************************/
#define HORNSAT_RAND 1000000
main ( int argc, char *argv[] ){
  if ( argc <2 ) goto ERROR;
  if ( argc == 3 && strchr( argv[2], 'q') ) HORNSAT_printflag = 0;
  else HORNSAT_printflag = 1;
  if ( argc == 3 && strchr( argv[2], 'v') ) HORNSAT_printflag |= 2;

  HORNSAT_init ();
  HORNSAT_load ( argv[1] );
  HORNSAT_output ();
  HORNSAT_end ();
  exit(0);

ERROR:;
  printf ("hornsat: [options] filename\n");
  printf ("Check satisfiablity of horn formula functions in filename\n");
  printf ("If the function is satisfiable, output the assignment of minimum truth\n");
  printf ("[options]\n");
  printf ("q: no output, v:verbose output\n"); 
  exit (1);
}





#endif


