/* Library of queue: spped priority implementation 
            12/Apr/2001   by Takeaki Uno  e-mail:uno@nii.jp, 
    homepage:   http://research.nii.ac.jp/~uno/index.html  */
/* This program is available for only academic use, basically.
   Anyone can modify this program, but he/she has to write down 
    the change of the modification on the top of the source code.
   Neither contact nor appointment to Takeaki Uno is needed.
   If one wants to re-distribute this code, do not forget to 
    refer the newest code, and show the link to homepage of 
    Takeaki Uno, to notify the news about the codes for the users.
   For the commercial use, please make a contact to Takeaki Uno. */

#ifndef _queue_c_
#define _queue_c_


#include"queue.h"
#include"stdlib2.c"

/* initialization, not fill the memory by 0 */
void QUEUE_init ( QUEUE *Q, int siz ){
  Q->s = 0;
  Q->t = 0;
  Q->end = siz+1;
  malloc2 ( Q->q, QUEUE_INT, siz+1, "QUEUE_init: Q->q");
}
void QUEUE_init_ ( QUEUE *Q, int siz, QUEUE_INT *buf ){
  Q->s = 0;
  Q->t = 0;
  Q->end = siz+1;
  Q->q = buf;
}

/* termination processing */
void QUEUE_end ( QUEUE *Q ){
  if ( Q->q ) free ( Q->q );
}

/* insert an element to the tail */
void QUEUE_ins_ ( QUEUE *Q, QUEUE_INT e ){
  Q->q[Q->t] = e;
  Q->t++;
}
void QUEUE_ins ( QUEUE *Q, QUEUE_INT e ){
  Q->q[Q->t] = e;
  QUEUE_INCREMENT ( *Q, Q->t );
  if ( Q->s == Q->t ) error ("QUEUE_ins: overflow");
}

/* insert an element to the head */
void QUEUE_ins_head_ ( QUEUE *Q, QUEUE_INT e ){
  Q->s--;
  Q->q[Q->s] = e;
}
void QUEUE_ins_head ( QUEUE *Q, QUEUE_INT e ){
  QUEUE_DECREMENT(*Q,Q->s);
  Q->q[Q->s] = e;
  if ( Q->s == Q->t ) error ("QUEUE_ins_head: underflow");
}

/* extract an element from the head, without checking underflow */
QUEUE_INT QUEUE_ext_ ( QUEUE *Q ){
  (Q->s)++;
  return ( Q->q[Q->s-1] );
}
QUEUE_INT QUEUE_ext ( QUEUE *Q ){
  QUEUE_INT e;
  if ( Q->s == Q->t ) error ("QUEUE_ext: empty queue");
  e = Q->q[Q->s];
  QUEUE_INCREMENT(*Q,Q->s);
  return ( e );
}

/* extract an element from the tail, without checking underflow */
QUEUE_INT QUEUE_ext_tail_ ( QUEUE *Q ){
  (Q->t)--;
  return ( Q->q[Q->t] );
}
QUEUE_INT QUEUE_ext_tail ( QUEUE *Q ){
  if ( Q->s == Q->t ) error ("QUEUE_ext_tail: empty queue");
  QUEUE_DECREMENT(*Q,Q->t);
  return ( Q->q[Q->t] );
}

/* remove the j-th element and replace it by the tail */
void QUEUE_rm_ ( QUEUE *Q, int j ){
  Q->t--;
  Q->q[j] = Q->q[Q->t];
}
void QUEUE_rm ( QUEUE *Q, int j ){
  if ( Q->s <= Q->t ){
    if ( j < Q->s || j >= Q->t ) error ("QUEUE_rm: j is out of queue");
  } else if ( j < Q->s && j >= Q->t ) error ("QUEUE_rm: j is out of queue");
  QUEUE_DECREMENT(*Q,Q->t);
  Q->q[j] = Q->q[Q->t];
}

/* remove the j-th element and replace it by the head */
void QUEUE_rm_head_ ( QUEUE *Q, int j ){
  Q->q[j] = Q->q[Q->s];
  Q->s++;
}
void QUEUE_rm_head ( QUEUE *Q, int j ){
  if ( Q->s <= Q->t ){
    if ( j < Q->s || j >= Q->t ) error ("QUEUE_rm: j is out of queue");
  } else if ( j < Q->s && j >= Q->t ) error ("QUEUE_rm: j is out of queue");
  Q->q[j] = Q->q[Q->s];
  QUEUE_INCREMENT(*Q,Q->s);
}

/* remove the j-th element and shift the following elements to fill the gap */
int QUEUE_rm_ele_ ( QUEUE *Q, QUEUE_INT e ){
  int i;
  QUEUE_F_LOOP ( *Q, i ){
    if ( Q->q[i] == e ){
      memcpy ( &(Q->q[i]), &(Q->q[i+1]), (Q->t-i-1)*sizeof(int));
      Q->t--;
      return (1);
    }
  }
  return (0);
}  
/* e Ał̓K؂Ȉʒuɑ} */
void QUEUE_ins_ele_ ( QUEUE *Q, QUEUE_INT e ){
  int i;
  QUEUE_INT ee;
  QUEUE_BE_LOOP_ ( *Q, i, ee ){
    if ( ee<e ) break;
    Q->q[i+1] = ee;
  }
  Q->q[i+1] = e;
  Q->t++;
}


/* Append Q2 to the tail of Q1. Q2 will not be deleted */
void QUEUE_concat_ ( QUEUE *Q1, QUEUE *Q2 ){
  memcpy ( &(Q1->q[Q1->t]), &(Q2->q[Q2->s]), QUEUE_LENGTH(*Q2)*sizeof(QUEUE_INT));
  Q1->t += Q2->t-Q2->s;
}
void QUEUE_concat ( QUEUE *Q1, QUEUE *Q2 ){
  int e = Q2->s;
  while ( e != Q2->t ){
    QUEUE_ins ( Q1, Q2->q[e] );
    QUEUE_INCREMENT(*Q2,e);
  }
}
/* Append Q2 to the tail of Q1. Q2 will be deleted */
void QUEUE_append_ ( QUEUE *Q1, QUEUE *Q2 ){
  QUEUE_concat_ ( Q1, Q2);
  QUEUE_RMALL ( *Q2 );
}
void QUEUE_append ( QUEUE *Q1, QUEUE *Q2 ){ // more improvement can be
  while ( Q2->s != Q2->t )
      QUEUE_ins ( Q1, QUEUE_ext( Q2 ) );
}

/* Append from j to jj th elements to the tail of Q1. Q2 will not be deleted */
void QUEUE_subconcat_ ( QUEUE *Q1, QUEUE *Q2, int j, int jj ){
  for ( ; j<=jj ; j++ ){
    Q1->q[Q1->t] = Q2->q[j];
    Q1->t++;
  }
}
void QUEUE_subconcat ( QUEUE *Q1, QUEUE *Q2, int j, int jj ){
  while (1){
    QUEUE_ins ( Q1, Q2->q[j] );
    if ( j == jj ) break;
    QUEUE_INCREMENT(*Q2,j);
  } 
}

/* initialize Q1 by length of Q2, and copy Q2 to Q1 */
void QUEUE_store_ ( QUEUE *Q1, QUEUE *Q2 ){
  QUEUE_init ( Q1, QUEUE_LENGTH(*Q2));
  QUEUE_concat_ ( Q1, Q2 );
}
void QUEUE_store ( QUEUE *Q1, QUEUE *Q2 ){
  QUEUE_init ( Q1, QUEUE_LENGTH(*Q2));
  QUEUE_concat ( Q1, Q2 );
}
/* copy Q2 to Q1 and delete Q2 */
void QUEUE_restore_ ( QUEUE *Q1, QUEUE *Q2 ){
  QUEUE_RMALL ( *Q1 );
  QUEUE_concat_ ( Q1, Q2 );
  QUEUE_end ( Q2 );
}
void QUEUE_restore ( QUEUE *Q1, QUEUE *Q2 ){
  QUEUE_RMALL ( *Q1 );
  QUEUE_concat ( Q1, Q2 );
  QUEUE_end ( Q2 );
}

/* copy Q2 to Q1 */
void QUEUE_cpy_ ( QUEUE *Q1, QUEUE *Q2 ){
  QUEUE_RMALL (*Q1);
  QUEUE_concat_ ( Q1, Q2 );
}
void QUEUE_cpy ( QUEUE *Q1, QUEUE *Q2 ){
  QUEUE_RMALL ( *Q1 );
  QUEUE_concat ( Q1, Q2 );
}
/* copy l elements of Q2 starting from s2 to the s1th position of Q1.
   size of Q1 is not increasing */
void QUEUE_subcpy_ ( QUEUE *Q1, int s1, QUEUE *Q2, int s2, int l ){
  memcpy ( &(Q1->q[s1]), &(Q2->q[s2]), (l-s2)*sizeof(QUEUE_INT));
}
void QUEUE_subcpy ( QUEUE *Q1, int s1, QUEUE *Q2, int s2, int l ){
  for ( ; s2!=l ; QUEUE_INCREMENT(*Q1,s1),QUEUE_INCREMENT(*Q2,s2) )
      Q1->q[s1] = Q2->q[s2];
  Q1->q[s1] = Q2->q[s2];
}

/* Q2𕡐Q1ɑBTCY݂͌Q1̒ɂ */
void QUEUE_dup_ ( QUEUE *Q1, QUEUE *Q2 ){
  QUEUE_init ( Q1, QUEUE_LENGTH(*Q2) );
  QUEUE_cpy_ ( Q1, Q2 );
}

/* merge Q1 and Q2 by insert all elements of Q2 to Q1 with deleting duplications. Both Q1 and Q2 have to be sorted in increasing order */
void QUEUE_merge_ ( QUEUE *Q1, QUEUE *Q2 ){
  int i=Q1->t-1, j=Q2->t-1, t=i+j-Q2->s+1;
  QUEUE_INT ei, ej;
  if ( i+1 == Q1->s || j+1 == Q2->s ){
    QUEUE_concat_ ( Q1, Q2 );
    return;
  }
  Q1->t = t+1;
  ei = Q1->q[i];
  ej = Q2->q[j];
  while (1){
    if ( ei > ej ){
      Q1->q[t] = ei;
      if ( i == Q1->s ){
        QUEUE_subcpy_ ( Q1, Q1->s, Q2, Q2->s, j );
        return;
      }
      i--;
      ei = Q1->q[i];
    } else {
      Q1->q[t] = ej;
      if ( j == Q2->s ) return;
      j--;
      ej = Q2->q[j];
    }
    t--;
  }
}
void QUEUE_merge ( QUEUE *Q1, QUEUE *Q2 ){
  int i=Q1->t, j=Q2->t;
  QUEUE_INT ei, ej;
  int t = (Q1->t + QUEUE_LENGTH(*Q2)-1) % Q1->end;
  if ( QUEUE_LENGTH(*Q1) + QUEUE_LENGTH(*Q2) >= Q1->end ){
    printf ("QUEUE_merge: overflow Q1->end=%d, Q1length=%d, Q2length=%d\n",
      Q1->end, QUEUE_LENGTH(*Q1), QUEUE_LENGTH(*Q2));
    exit (1);
  }
  if ( i == Q1->s || j == Q2->s ){
    QUEUE_concat ( Q1, Q2 );
    return;
  }

  Q1->t = t;
  QUEUE_DECREMENT(*Q1,i);
  QUEUE_DECREMENT(*Q2,j);
  ei = Q1->q[i];
  ej = Q2->q[j];
  while (1){
    if ( ei > ej ){
      Q1->q[t] = ei;
      if ( i == Q1->s ){
        QUEUE_subcpy ( Q1, Q1->s, Q2, Q2->s, (j+Q2->end-Q2->s)%Q2->end );
        return;
      }
      QUEUE_DECREMENT(*Q1,i);
      ei = Q1->q[i];
    } else {  
      Q1->q[t] = ej;
      if ( j == Q2->s ) return;
      QUEUE_DECREMENT(*Q2,j);
      ej = Q2->q[j];
    }
    QUEUE_DECREMENT(*Q1,t);
  }
}

/* delete all elements of Q1 included in Q2.
 both Q1 and Q2 have to be sorted in increasing order */
/* Q1̗vfQ2̗vfSĎ菜B҂ƂɃ\[gĂ邱 */
void QUEUE_minus_ ( QUEUE *Q1, QUEUE *Q2 ){
  int i=Q1->s, i2 = Q2->s, ii=Q1->s;
  while ( i != Q1->t && i2 != Q2->t){
    if ( Q1->q[i] > Q2->q[i2] ) i2++;
    else {
      if ( Q1->q[i] < Q2->q[i2] ){
        Q1->q[ii] = Q1->q[i];
        ii++;
      }
      i++;
    }
  }
  while ( i != Q1->t ){
    Q1->q[ii] = Q1->q[i];
    i++;
    ii++;
  }
  Q1->t = ii;
}
void QUEUE_minus ( QUEUE *Q1, QUEUE *Q2 ){
  int i=Q1->s, i2 = Q2->s, ii=Q1->s;
  while ( i != Q1->t && i2 != Q2->t){
    if ( Q1->q[i] > Q2->q[i2] ) QUEUE_INCREMENT ( *Q2, i2);
    else {
      if ( Q1->q[i] < Q2->q[i2] ){
        Q1->q[ii] = Q1->q[i];
        QUEUE_INCREMENT ( *Q1, ii);
      }
      QUEUE_INCREMENT ( *Q1, i);
    }
  }
  while ( i != Q1->t ){
    Q1->q[ii] = Q1->q[i];
    QUEUE_INCREMENT ( *Q1, i);
    QUEUE_INCREMENT ( *Q1, ii);
  }
  Q1->t = ii;
}

/* Delete all elements of Q1 which are not included in Q2. 
 both Q1 and Q2 have to be sorted in increasing order */
void QUEUE_and_ ( QUEUE *Q1, QUEUE *Q2 ){
  int i=Q1->s, i2 = Q2->s, ii=Q1->s;
  while ( i != Q1->t && i2 != Q2->t){
    if ( Q1->q[i] > Q2->q[i2] ) i2++;
    else {
      if ( Q1->q[i] == Q2->q[i2] ){
        Q1->q[ii] = Q1->q[i];
        ii++;
      }
      i++;
    }
  }
  Q1->t = ii;
}
void QUEUE_and ( QUEUE *Q1, QUEUE *Q2 ){
  int i=Q1->s, i2 = Q2->s, ii=Q1->s;
  while ( i != Q1->t && i2 != Q2->t){
    if ( Q1->q[i] > Q2->q[i2] ) QUEUE_INCREMENT ( *Q2, i2);
    else {
      if ( Q1->q[i] == Q2->q[i2] ){
        Q1->q[ii] = Q1->q[i];
        QUEUE_INCREMENT ( *Q1, ii);
      }
      QUEUE_INCREMENT ( *Q1, i);
    }
  }
  Q1->t = ii;
}

/* sort elements in Q. it is not adapt for the change of the type QUEUE_INT */
void QUEUE_qsort ( QUEUE *Q ){

#if QUEUE_INT==int
  qsort_int ( Q->q+Q->s, Q->t-Q->s );
#else
#if QUEUE_INT==short
  qsort_short ( Q->q+Q->s, Q->t-Q->s );
#endif
#endif

}
void QUEUE_qsort_ ( QUEUE *Q ){

#if QUEUE_INT==int
  qsort_int_ ( Q->q+Q->s, Q->t-Q->s );
#else
#if QUEUE_INT==short
  qsort_short_ ( Q->q+Q->s, Q->t-Q->s );
#endif
#endif

}
/* insertion sort */
void QUEUE_sort ( QUEUE *Q ){
  int i = Q->s, j, jj;
  QUEUE_INT e;
  if ( i== Q->t ) return;
  QUEUE_INCREMENT(*Q,i);
  for ( ; i!=Q->t ; QUEUE_INCREMENT(*Q,i) ){
    e=Q->q[i]; 
    j=i; 
    while (1){
      jj = j;
      QUEUE_DECREMENT(*Q,j);
      if ( Q->q[j] <= e ) { Q->q[jj] = e; break; }
      Q->q[jj] = Q->q[j];
      if ( j == Q->s) { Q->q[j] = e; break; }
    }
  }
}


/* print a queue */
void QUEUE_print ( QUEUE *Q ){
  int i;
  for ( i=Q->s ; i!=Q->t ; ){
    printf ("%d,",Q->q[i]);
    QUEUE_INCREMENT(*Q,i);
  }
  printf ("\n");
}
/* permutation version */
void QUEUE_perm_print ( QUEUE *Q, int *q ){
  int i;
  for ( i=Q->s ; i!=Q->t ; ){
    printf ("%d,",q[Q->q[i]]);
    QUEUE_INCREMENT(*Q,i);
  }
  printf ("\n");
}
void QUEUE_printn ( QUEUE *Q ){
  int i;
  for ( i=Q->s ; i!=Q->t ; ){
    printf ("%d,",Q->q[i]);
    QUEUE_INCREMENT(*Q,i);
  }
}
void QUEUE_perm_printn ( QUEUE *Q, int *q ){
  int i;
  for ( i=Q->s ; i!=Q->t ; ){
    printf ("%d,",q[Q->q[i]]);
    QUEUE_INCREMENT(*Q,i);
  }
}
void QUEUE_print_ ( QUEUE *Q ){
  int i;
  printf("s=%d,t=%d: ", Q->s, Q->t);
  for ( i=Q->s ; i!=Q->t ; ){
    printf ("%d,",Q->q[i]);
    QUEUE_INCREMENT(*Q,i);
  }
  printf ("\n");
}

void QUEUE_print__ ( QUEUE *Q ){
  int i;
  printf("s=%d,t=%d: ", Q->s, Q->t);
  for ( i=Q->s ; i!=Q->t ; i++ )
    printf ("%d,",Q->q[i]);
  printf ("\n");
}

/*
main (){
  QUEUE Q1, Q2, Q3;
  QUEUE_init ( &Q1, 10 );
  QUEUE_init ( &Q2, 10 );
  QUEUE_init ( &Q3, 10 );
  QUEUE_ins ( &Q1, 5 );
  QUEUE_ins ( &Q1, 7 );

  QUEUE_ins_ ( &Q2, 5 );
  QUEUE_ins_ ( &Q2, 4 );
  QUEUE_ins_ ( &Q2, 6 );
  QUEUE_merge_ ( &Q3, &Q1, &Q2 );
  QUEUE_print_ (&Q1); 
  QUEUE_print_ (&Q2); 
  QUEUE_print_ (&Q3); 
}
*/

/*
main (){
  QUEUE Q1, Q2, Q3;
  int i;
  QUEUE_init ( &Q1, 100 );
  QUEUE_init ( &Q2, 100 );
  for ( i=0 ; i<10 ; i++ ){
    QUEUE_ins ( &Q1, rand()%50 );
    QUEUE_ins_head ( &Q1, rand()%50 );
  }
  for ( i=0 ; i<5 ; i++ ){
    QUEUE_ins ( &Q2, rand()%50 );
    QUEUE_ins_head ( &Q2, rand()%50 );
  }

  QUEUE_print_ (&Q1); 
  QUEUE_print_ (&Q2); 
  QUEUE_sort ( &Q1 );
  QUEUE_sort ( &Q2 );
  QUEUE_print_ (&Q1); 
  QUEUE_print_ (&Q2); 
  QUEUE_merge ( &Q1, &Q2); 
  QUEUE_print_ (&Q1); 
}
*/


#endif
