/* Library of queue: spped priority implementation 
            12/Apr/2001   by Takeaki Uno  e-mail:uno@nii.jp, 
    homepage:   http://research.nii.ac.jp/~uno/index.html  */
/* This program is available for only academic use, basically.
   Anyone can modify this program, but he/she has to write down 
    the change of the modification on the top of the source code.
   Neither contact nor appointment to Takeaki Uno is needed.
   If one wants to re-distribute this code, do not forget to 
    refer the newest code, and show the link to homepage of 
    Takeaki Uno, to notify the news about the codes for the users.
   For the commercial use, please make a contact to Takeaki Uno. */

#ifndef _queue_h_
#define _queue_h_

#include"stdlib2.h"

#ifndef QUEUE_INT
#define QUEUE_INT int    // define the type before if change is needed 
#define QUEUE_INT_INT    // comment out if QUEUE_INT is "short"
#endif

typedef struct {
  int s;  // start position
  int t;  // end position+1
  int end;  // the length of the array
  QUEUE_INT *q;  // pointer to the array
} QUEUE;

/* QUEUE stores at most end-1 elements. Overflow occurs after inserting end-1 elements */

#define QUEUE_INCREMENT(Q,i) ((i)=((i)>=(Q).end-1)?0:(i)+1)
#define QUEUE_DECREMENT(Q,i) ((i)=(i)==0?(Q).end-1:(i)-1)
#define QUEUE_LENGTH(Q) (((Q).t-(Q).s+(Q).end)%(Q).end)
#define QUEUE_LENGTH_(Q) ((Q).t-(Q).s)

/* macro for loop w.r.t., QUEUE */
#define QUEUE_F_LOOP(Q,i)  for((i)=(Q).s;(i)!=(Q).t;((i)=((i)>=(Q).end-1)?0:(i)+1))
#define QUEUE_F_LOOP_(Q,i)  for((i)=(Q).s;(i)<(Q).t;(i)++)
#define QUEUE_FE_LOOP(Q,i,x)  for((i)=(Q).s,x=(Q).q[i];(i)!=(Q).t;((i)=((i)>=(Q).end-1)?0:(i)+1),x=(Q).q[i])
#define QUEUE_FE_LOOP_(Q,i,x)  for((i)=(Q).s,x=(Q).q[i];(i)<(Q).t;(i)++,x=(Q).q[i])
#define QUEUE_B_LOOP(Q,i)  for((i)=(Q).t==0?(Q).end-1:(Q).t-1;(i)!=(Q).s;(i)=(i)==0?(Q).end-1:(i)-1)
#define QUEUE_B_LOOP_(Q,i)  for((i)=(Q).t-1;(i)>=(Q).s;(i)--)
#define QUEUE_BE_LOOP(Q,i,x)  for((i)=(Q).t==0?(Q).end-1:(Q).t-1,x=(Q).q[i];(i)!=(Q).s;(i)=(i)==0?(Q).end-1:(i)-1,x=(Q).q[i])
#define QUEUE_BE_LOOP_(Q,i,x)  for((i)=(Q).t-1,x=(Q).q[i];(i)>=(Q).s;(i)--,x=(Q).q[i])

#define QUEUE_RMALL(Q) ((Q).t=(Q).s)
#define QUEUE_RMALL_(Q) ((Q).t=0)
#define QUEUE_HEAD(Q) ((Q).q[(Q).s])
#define QUEUE_TAIL_(Q) ((Q).q[(Q).t-1])


/* initialization, not fill the memory by 0 */
void QUEUE_init ( QUEUE *Q, int siz );
void QUEUE_init_ ( QUEUE *Q, int siz, QUEUE_INT *buf );

/* termination processing */
void QUEUE_end ( QUEUE *Q );

/* insert an element to the tail/head */
void QUEUE_ins_ ( QUEUE *Q, QUEUE_INT e );
void QUEUE_ins ( QUEUE *Q, QUEUE_INT e );
void QUEUE_ins_head_ ( QUEUE *Q, QUEUE_INT e );
void QUEUE_ins_head ( QUEUE *Q, QUEUE_INT e );

/* extract an element from the head/tail, without checking the underflow */
QUEUE_INT QUEUE_ext_ ( QUEUE *Q );
QUEUE_INT QUEUE_ext ( QUEUE *Q );
QUEUE_INT QUEUE_ext_tail_ ( QUEUE *Q );
QUEUE_INT QUEUE_ext_tail ( QUEUE *Q );

/* remove the j-th element and replace it by the tail/head or shift */
void QUEUE_rm_ ( QUEUE *Q, int j );
void QUEUE_rm ( QUEUE *Q, int j );
void QUEUE_rm_head_ ( QUEUE *Q, int j );
void QUEUE_rm_head ( QUEUE *Q, int j );
int QUEUE_rm_ele_ ( QUEUE *Q, QUEUE_INT e );

/* Append Q2 to the tail of Q1. Q2 will (not) be deleted */
void QUEUE_append_ ( QUEUE *Q1, QUEUE *Q2 );
void QUEUE_append ( QUEUE *Q1, QUEUE *Q2 );
void QUEUE_concat_ ( QUEUE *Q1, QUEUE *Q2 );
void QUEUE_concat ( QUEUE *Q1, QUEUE *Q2 );

/* Append from j to jj th elements to the tail of Q1. Q2 will not be deleted */
void QUEUE_subconcat_ ( QUEUE *Q1, QUEUE *Q2, int j, int jj );
void QUEUE_subconcat ( QUEUE *Q1, QUEUE *Q2, int j, int jj );

/* initialize Q1 by length of Q2, and copy Q2 to Q1 */
void QUEUE_store_ ( QUEUE *Q1, QUEUE *Q2 );
void QUEUE_store ( QUEUE *Q1, QUEUE *Q2 );
/* copy Q2 to Q1 and delete Q2 */
void QUEUE_restore_ ( QUEUE *Q1, QUEUE *Q2 );
void QUEUE_restore ( QUEUE *Q1, QUEUE *Q2 );

/* copy Q2 to Q1 */
void QUEUE_cpy_ ( QUEUE *Q1, QUEUE *Q2 );
void QUEUE_cpy ( QUEUE *Q1, QUEUE *Q2 );
/* copy l elements of Q2 starting from s2 to the s1th position of Q1.
   size of Q1 is not increasing */
void QUEUE_subcpy_ ( QUEUE *Q1, int s1, QUEUE *Q2, int s2, int l );
void QUEUE_subcpy ( QUEUE *Q1, int s1, QUEUE *Q2, int s2, int l );

/* merge/minum/intersection of Q1 and Q2, and set Q1 to it.
 Both Q1 and Q2 have to be sorted in increasing order */
void QUEUE_merge_ ( QUEUE *Q1, QUEUE *Q2 );
void QUEUE_merge ( QUEUE *Q1, QUEUE *Q2 );
void QUEUE_minus_ ( QUEUE *Q1, QUEUE *Q2 );
void QUEUE_minus ( QUEUE *Q1, QUEUE *Q2 );
void QUEUE_and_ ( QUEUE *Q1, QUEUE *Q2 );
void QUEUE_and ( QUEUE *Q1, QUEUE *Q2 );
 
  /* sort of QUEUE, not adapt to the change of type QUEUE_INT */
void QUEUE_sort_ ( QUEUE *Q );
/* insertion sort */
void QUEUE_sort ( QUEUE *Q );

  /* print */
void QUEUE_print ( QUEUE *Q );
void QUEUE_print_ ( QUEUE *Q );


#endif
