/* library for standard macros and functions */
/* by Takeaki Uno 2/22/2002, e-mail: uno@nii.jp
    homepage:   http://research.nii.ac.jp/~uno/index.html  */
/* This program is available for only academic use, basically.
   Anyone can modify this program, but he/she has to write down 
    the change of the modification on the top of the source code.
   Neither contact nor appointment to Takeaki Uno is needed.
   If one wants to re-distribute this code, do not forget to 
    refer the newest code, and show the link to homepage of 
    Takeaki Uno, to notify the news about the codes for the users.
   For the commercial use, please make a contact to Takeaki Uno. */

#ifndef _stdlib2_c_
#define _stdlib2_c_

#include"stdlib2.h"

/*  error routine  */
void error ( char *mes ){
  printf ("error: %s\n", mes);
  exit (1);
}




/********************  file I/O routines  ********************************/

int FILE_err;  /*  signals  0: for normal termination
                   1: read a number, then encountered a newline,
                   2: read a number, then encountered the end-of-file
                   5: read no number, and encountered a newline
                   6: read no number, and encountered the end-of-file */

/* count the number of integers in the flie */
/* set *rows:=#rows, *maxn:=maximum among the integers read,
       *minn:=minimum among the integers read  */
int file_count_number ( char * fname, int *rows, int *maxn, int *minn ){
  FILE *fp;
  int x=0, nn=0, cr=1;
  fopen2r ( fp, fname, "file_count_number");
  *rows=0; *maxn=-INTHUGE; *minn=INTHUGE;
  while (1) {
    x = file_read_int (fp);
    if ( (FILE_err&4)==0 ){
      if ( x>*maxn ) *maxn = x;
      if ( x<*minn ) *minn = x;
      nn++;
      cr=0;
    }
    if ( FILE_err&2 ){
      if ( cr==0 ) (*rows)++;
      break;
    }
    if ( FILE_err&1 ){ (*rows)++; cr=1; }
  }
  fclose (fp);
  return (nn);
}


/* Read an integer from the file and return it,
    with read through the non-numeric letters.
   If it reaches to the end-of-file, then set FILE_err=2, if it reads a 
   newline, then set FILE_err=1.
   If read either the end-of-file or newline before reading an integer,
   return -1 */
int file_read_int (FILE *fp){
  int item;
  char ch;
  FILE_err = 0;
  do {
    ch = getc(fp);
    if ( ch == '\n' ){ FILE_err = 5; return (-1); }
    if ( feof(fp) ){ FILE_err = 6; return (-1); }
  } while ( ch<'0' || ch>'9' );

  for ( item=(int)(ch-'0') ; 1 ; item=item*10 +(int)(ch-'0') ){
    ch = getc(fp);
    if ( ch == '\n' ){ FILE_err = 1; return (item); }
    if ( feof(fp) ){ FILE_err = 2; return (item); }
    if ( (ch < '0') || (ch > '9')) return (item );
  }
}

/* Read integers until encountering either newline or the end-of-file,
   and store the integers to the given array */
/* return the number of intergers read */
/* do not check the overflow of the array */
int file_read_int_array (int *a, FILE *fp){
  int x, t=0;
  do {
    x = file_read_int (fp);
    if ( FILE_err&4 ) break;
    a[t] = x;
    t++;
  } while (FILE_err==0);
  return (t);
}




/******************* integer array routines **************************/

/* duplication of array of integers */
void *array_dup ( void *a, int end, int unit ){
  char *p;
  malloc2 ( p, char, end*unit, "array_malloc_copy: p" );
  memcpy ( p, a, unit*end );
  return ( p );
}

/* allocate an array of integers of size "end", and set each integer to "c" */
int *intarray_malloc_const ( int end, int c ){
  int i, *a;
  malloc2 ( a, int, end, "intarray_malloc_const: a");
  for ( i=0 ; i<end ; i++ ) a[i]=c;
  return (a);
}
/* return the index having the maximum value, in an integer array */
int intarray_max ( int *a, int end ){
  int i, ii=0, m = -INTHUGE;
  for ( i=0 ; i<end ; i++ )
      if ( a[i] > m ){ m=a[i]; ii=i;}
  return (ii);
}
/* return the index having the minimum value, in an integer array */
int intarray_min ( int *a, int end ){
  int i, ii=0, m = INTHUGE;
  for ( i=0 ; i<end ; i++ )
      if ( a[i] < m ){ m=a[i]; ii=i;}
  return (ii);
}
/* print integer array */
void intarray_print ( int *a, int end ){
  int i;
  for ( i=0 ; i<end ; i++ ) printf ("%d,", a[i] );
  printf ("\n");
}
/* output/input integer array to/from a file */
/* void fprint_int_array ( FILE *fp, int *a, int end ){
  int i;
  for ( i=0 ; i<end ; i++ ) fprintf (fp, "%d,", a[i] );
  fprintf (fp, "\n");
}*/ 
void intarray_fprint ( char *filename, int *a, int end ){
  int i;
  FILE *fp;
  fopen2r ( fp, filename, "fscan_int_array");
  for ( i=0 ; i<end ; i++ ) fprintf (fp, "%d,", a[i] );
  fclose (fp);
}
int *intarray_fscan ( char *filename ){
  int i, *a, c=0;
  FILE *fp;
  fopen2r ( fp, filename, "fscan_int_array");
  while ( !feof(fp) ){
    fscanf (fp, "%d,", &i );
    c++;
  }
  fseek ( fp, 0, SEEK_SET );
  malloc2 ( a, int, c, "fscan_int_array: a");
  for ( i=0 ; i<c ; i++ )
      fscanf (fp, "%d,", &a[i]);
  fclose ( fp );
  return (a);
}

/******************************* permutation routines ****************/
/* permutation is given by an integer array  */


/* return the pointer to an ??? permutation */
int *init_perm ( int end ){
  int i, *p;
  malloc2 ( p, int, end, "init_perm: p" );
  for ( i=0 ; i<end ; i++ ) p[i] = i;
  return ( p );
}

/* return the pointer to an inverse permutation of the given permutation */
int *inverse_perm ( int *perm, int end ){
  int i, *p;
  malloc2 ( p, int, end, "inverse_perm: p" );
  for ( i=0 ; i<end ; i++ ) p[i] = -1;
  for ( i=0 ; i<end ; i++ )
      if ( perm[i]>=0 && perm[i]<end ) p[perm[i]] = i; 
  return ( p );
}

/* sort an array of size "siz", composed of a structure of size "unit" byte
 in the order of perm */
/* use temporary memory of siz*unit byte */
void perm_struct ( void *a, int unit, int *perm, int siz ){
  int i;
  char *s, *ss = a;
  malloc2 ( s, char, unit*siz, "perm_struct: s" );
  memcpy ( s, ss, unit*siz );
  for ( i=0 ; i<siz ; i++ )
    memcpy ( ss + unit*perm[i], s + unit*i, unit );
  free ( s );
}


/* SLIST:very simple one-sided list */
void SLIST_init ( int *l, int num, int siz ){
  int i;
  malloc2 ( l, int, num+siz, "SLIST_init: l");
  for ( i=num ; i<num+siz ; i++ ) l[i] = -1;
}
void SLIST_end ( int *l ){
  free (l);
}
#define SLIST_INS(l,m,e) (l[e]=l[m],l[m]=e);



/* basic comparison functions for quick sort */
int qsort_int_cmp ( const void *x, const void *y ){
  if ( *((int *)x) < *((int *)y) ) return (-1);
  else return ( *((int *)x) > *((int *)y) );
}
int qsort_int_cmp_ ( const void *x, const void *y ){
  if ( *((int *)x) > *((int *)y) ) return (-1);
  else return ( *((int *)x) < *((int *)y) );
}
int qsort_double_cmp ( const void *x, const void *y ){
  if ( *((double *)x) < *((double *)y) ) return (-1);
  else return ( *((double *)x) > *((double *)y) );
}
int qsort_char_cmp ( const void *x, const void *y ){
  if ( *((char *)x) < *((char *)y) ) return (-1);
  else return ( *((char *)x) > *((char *)y) );
}
int qsort_short_cmp ( const void *x, const void *y ){
  if ( *((short *)x) < *((short *)y) ) return (-1);
  else return ( *((short *)x) > *((short *)y) );
}
int qsort_short_cmp_ ( const void *x, const void *y ){
  if ( *((short *)x) > *((short *)y) ) return (-1);
  else return ( *((short *)x) < *((short *)y) );
}

/* basic quick sorts: sort the elements from 0 to siz -1 
  for sorting from "k"-th to "l"-th elements of an array "a", 
  parameter has to be "a+k" and "l-k"  */
void qsort_int ( int *a, int siz ){
    qsort ( a, siz, sizeof(int), qsort_int_cmp );
}
void qsort_int_ ( int *a, int siz ){
    qsort ( a, siz, sizeof(int), qsort_int_cmp_ );
}
void qsort_double ( double *a, int siz ){
    qsort ( a, siz, sizeof(double), qsort_double_cmp );
}
void qsort_char ( char *a, int siz ){
    qsort ( a, siz, sizeof(char), qsort_char_cmp );
}
void qsort_int2 ( int *a, int siz ){
    qsort ( a, siz, sizeof(int)*2, qsort_int_cmp );
}
void qsort_short ( short *a, int siz ){
    qsort ( a, siz, sizeof(short), qsort_short_cmp );
}
void qsort_short_ ( short *a, int siz ){
    qsort ( a, siz, sizeof(short), qsort_short_cmp_ );
}
/* sort array of structure of size "unit", or etc. by the integers
  located at the top of them */
/* in the decreasing order if flag !=0 */
void qsort_int_struct ( int *a, int siz, int unit, int flag ){  
  if ( flag ) qsort ( a, siz, unit, qsort_int_cmp_ );
  else qsort ( a, siz, unit, qsort_int_cmp );
}
/* return the permutation of the result of the sorting the integers 
 of an array of structures of size "unit".
  give the pointer to the integer member, to be sorted, of the first element */
/* use temporary memory of size "siz*2*sizeof(int)" byte */
/* in the decreasing order if flag !=0 */
int *qsort_struct_perm ( void *a, int siz, int unit, int flag ){  
  int *s, i, *perm;
  malloc2 ( s, int, siz*2, "qsort_int2_perm: s" );
  malloc2 ( perm, int, siz, "qsort_int2_perm: perm" );
  for ( i=0 ; i<siz ; i++ ){
    s[i*2] = *((int *)( ((char *)a)+unit*i ));
    s[i*2+1] = i;
  }
  qsort_int_struct ( s, siz, sizeof(int)*2, flag );
  for ( i=0 ; i<siz ; i++ ) perm[s[i*2+1]] = i;
  free ( s );
  return ( perm );
}
/* a version of given the "perm" array, already allocated  */
void qsort_struct_perm_ ( void *a, int siz, int unit, int *perm, int flag ){  
  int *s, i, *p;
  malloc2 ( s, int, siz*2, "qsort_int2_perm: s" );
  for ( i=0 ; i<siz ; i++ ){
    s[i*2] = *((int *)( ((char *)a)+unit*i ));
    s[i*2+1] = i;
  }
  qsort_int_struct ( s, siz, sizeof(int)*2, flag );
  p = array_dup ( perm, siz, sizeof(int) );
  for ( i=0 ; i<siz ; i++ ) perm[s[i*2+1]] = p[i];
  free ( s );
  free ( p );
}


/* radix sort for array of structures, by their integer members 
  ranging from mm to m */
/* sort array "perm" according to (int/void*) array "a".
   if perm==NULL, allocate memory and for perm */
/* return the permutation array of the result of the sorting
   in the decreasing order if flag!=0 */
int *radix_sort ( void *a, int siz, int unit, int mm, int m, int *perm, int flag ){ 
  int *ll, *l, k, i, t;
  l = intarray_malloc_const ( m-mm, -1 );
  malloc2 ( ll, int, siz, "radix_sort: ll");
  for ( i=0 ; i<siz ; i++ ){
    k = (*((int *)(((char *)a) + unit*i ))) - mm;
    ll[i] = l[k];
    l[k] = i;
  }
  if ( perm ){
    for ( k=0,i=0 ; k<m-mm ; k++ ){
      while ( l[k] >= 0 ){
        t = l[k];
        l[k] = ll[t];
        ll[t] = perm[i];
        i++;
      }
    }
    memcpy ( perm, ll, sizeof(int)*siz );
    free ( ll );
    free ( l );
    return ( perm );
  } else {
    for ( k=0,i=0 ; k<m-mm ; k++ ){
      while ( l[k] >= 0 ){
        t = l[k];
        l[k] = ll[t];
        ll[t] = i;
        i++;
      }
    }
    free ( l );
    return ( ll );
  }
}



#endif


/****************************************** 
/*  ==== terminology for comments  ====
  range check: to check the input parameter is valid, or in the valid range.
  If a function does not have this, its comment has "no range check"
  */
   
/*  ====  rules for the name of functions and routines  ====
  init: initialization for an object, structure, etc. memory is allocated 
   if needed.
  end: compared to init, termination of structures, etc.
   free allocated memory if it exists, but not free itself.
   different from ordinary new, create, del, free.

  cpy: copy an object without allocating memory
  dup: make a duplication of an object with allocating new memory

  new: new. allocate memory for new object. also used for re-allocation from 
   the list of deleted objects
  del: delete. free memory, or insert it to the list of deleted objects

  ins : insert. insert an element (active, not deleted) to an object, possible
     at the given position.
  out : extract. extract an element, possibly specified, from an object.
        it will be not deleted.
  rm  : extract, and delete
  rmall: delete all (specified) elements of an object
  mk  : make. new+insB
  mv  : move. move the elements from an object to another,
         or change the position.

  update : update an object, possibly of specified position, to the exact,
           or to the current one.
  chg    : change the status of an object to the specified one.
  
  prv: point the previous element
  nxt: point the next element
  end: the maximum size (allocated size) of an array, etc.
  num: the (current) number of elements in an array, etc.
  kth: for the use of "first k object"
  tkth: for the use of "first k object from the end". to kth.
  rnd: random
  print: print structure and etc.
  find: search or find an specified element from the set of structures, etc.
*/

/*  ==== rules for the name of variables ====
  - use i or j for the counter in loops
  - use e for elements
  - use k for the specified rank
  - use f or flag for flagment
  - use m for maximum value or minimum value
  - use c for counters
*/


