/* library for standard macros and functions 
 by Takeaki Uno 2/22/2002   e-mail: uno@nii.jp
    homepage:   http://research.nii.ac.jp/~uno/index.html  */
/* This program is available for only academic use, basically.
   Anyone can modify this program, but he/she has to write down 
    the change of the modification on the top of the source code.
   Neither contact nor appointment to Takeaki Uno is needed.
   If one wants to re-distribute this code, do not forget to 
    refer the newest code, and show the link to homepage of 
    Takeaki Uno, to notify the news about the codes for the users.
   For the commercial use, please make a contact to Takeaki Uno. */

#ifndef _stdlib2_h_
#define _stdlib2_h_

#include<stdlib.h>
#include<stdio.h>
#include<math.h>

/* comment out the following line if no error check is needed */
#define ERROR_CHECK


#ifndef NULL
#define NULL 0
#endif

#define UINTHUGE 4000000000
#define INTHUGE 2000000000
#define USHORTHUGE 32767
#define SHORTHUGE 65535
#define DOUBLEHUGE 999999999999999999999999999999.9

#define ISEQUAL_VALUE   0.0000001
#define ISEQUAL_VALUE2  0.00001

/*  random  */
#define rnd(a)    (random()%(a))
#define prob(a)   ((random()%65536)<(int)((a)*65536.0))

/* macros for allocating memory with exiting if an error occurs */
#define   malloc2(f,a,b,c)     if(!(f=(a *)malloc(sizeof(a)*(b)))){printf("memory allocation error %s (%d byte)\n",c,sizeof(a)*(b));exit(1);}
#define   realloc2(f,a,b,c)     if(!(f=(a *)realloc(f,sizeof(a)*(b)))){printf("memory error, %s (%d byte)\n",c,sizeof(a)*(b));exit(1);}
#define xalloc(a,b)	(a *)malloc((b)*sizeof(a))
#define xcalloc(a,b)	(a *)calloc((b)*sizeof(a))

#define   free2(a)  if(a){free(a);a=NULL;}

/* a macro for open files with exiting if an error occurs */
#define   fopen2(f,a,c)     if(!(f=fopen(a,"a+"))){printf("file open error: %s, file name %s\n",c,a);exit(1);}
#define   fopen2r(f,a,c)     if(!(f=fopen(a,"r+"))){printf("file open error: %s, file name %s\n",c,a);exit(1);}
#define   fopen2w(f,a,c)     if(!(f=fopen(a,"w+"))){printf("file open error: %s, file name %s\n",c,a);exit(1);}


#define MARK      1
#define UNMARK    0
#define TRUE      1
#define FALSE     0

/*  equal/inequal with allowing numerical error for double  */
#define ISEQUAL(a,b)	(a-b<ISEQUAL_VALUE&&b-a<ISEQUAL_VALUE)
#define ISGREAT(a,b)    (a-b>ISEQUAL_VALUE)
#define ISLESS(a,b) 	(b-a>ISEQUAL_VALUE)

/*  macro for getting maximum/minimum of two values  */
#define MAX(a,b)      ((a)>(b)?a:b)
#define MIN(a,b)      ((a)<(b)?a:b)

/*  error routine  */
void error ( char *mes );

/* swap macro for integer, double, char, and pointer */
int swap_tmp_int;
double swap_tmp_double;
char swap_tmp_char;
void *swap_tmp_pointer;
#define SWAP_INT(a,b)  (swap_tmp_int=a,a=b,b=swap_tmp_int)
#define SWAP_DOUBLE(a,b)  (swap_tmp_double=a,a=b,b=swap_tmp_double)
#define SWAP_CHAR(a,b)  (swap_tmp_char=a,a=b,b=swap_tmp_char)
#define SWAP_PNT(a,b)  (swap_tmp_char=(void *)a,a=b,b=swap_tmp_char)


/*********************************************************/


/* count the number of integers in the flie */
/* set *rows:=#rows, *maxn:=maximum among the integers read,
       *minn:=minimum among the integers read  */
int file_count_number ( char * fname, int *rows, int *maxn, int *minn );

/* Read an integer from the file and return it,
    with read through the non-numeric letters.
   If it reaches to the end-of-file, then set FILE_err=2, if it reads a 
   newline, then set FILE_err=1.
   If read either the end-of-file or newline before reading an integer,
   return -1 */
int file_read_int (FILE *fp);

/* Read integers until encountering either newline or the end-of-file,
   and store the integers to the given array */
/* return the number of intergers read */
/* do not check the overflow of the array */
int file_read_int_array (int *a, FILE *fp);

/******************* integer array routines **************************/

/* duplication of array of integers */
void *array_dup ( void *a, int end, int unit );
/* allocate an array of integers of size "end", and set each integer to "c" */
int *intarray_malloc_const ( int end, int c );
/* return the index having the maximum/minimum value, in an integer array */
int intarray_max ( int *a, int end );
int intarray_min ( int *a, int end );
/* print integer array */
void intarray_print ( int *a, int end );
/* output/input integer array to/from a file */
void intarray_fprint ( char *filename, int *a, int end );
int *intarray_fscan ( char *filename );

/******************************* permutation routines ****************/
/* permutation is given by an integer array  */

/* return the pointer to an ??? permutation */
int *init_perm ( int end );

/* return the pointer to an inverse permutation of the given permutation */
int *inverse_perm ( int *perm, int end );

/* sort an array of size "siz", composed of a structure of size "unit" byte
 in the order of perm */
/* use temporary memory of siz*unit byte */
void perm_struct ( void *a, int unit, int *perm, int siz );

/* SLIST:very simple one-sided list */
void SLIST_init ( int *l, int num, int siz );
void SLIST_end ( int *l );
#define SLIST_INS(l,m,e) (l[e]=l[m],l[m]=e);



/* basic comparison functions for quick sort */
int qsort_int_cmp ( const void *x, const void *y );
int qsort_int_cmp_ ( const void *x, const void *y );
int qsort_double_cmp ( const void *x, const void *y );
int qsort_char_cmp ( const void *x, const void *y );
int qsort_short_cmp ( const void *x, const void *y );
int qsort_short_cmp_ ( const void *x, const void *y );
/* basic quick sorts: sort the elements from 0 to siz -1 
  for sorting from "k"-th to "l"-th elements of an array "a", 
  parameter has to be "a+k" and "l-k"  */
void qsort_int ( int *a, int siz );
void qsort_int_ ( int *a, int siz );
void qsort_double ( double *a, int siz );
void qsort_char ( char *a, int siz );
void qsort_short ( short *a, int siz );
void qsort_short_ ( short *a, int siz );
void qsort_int2 ( int *a, int siz );
/* sort array of structure of size "unit", or etc. by the integers
  located at the top of them */
/* in the decreasing order if flag !=0 */
void qsort_int_struct ( int *a, int siz, int unit, int flag );

/* return the permutation of the result of the sorting the integers 
 of an array of structures of size "unit".
  give the pointer to the integer member, to be sorted, of the first element */
/* use temporary memory of size "siz*2*sizeof(int)" byte */
/* in the decreasing order if flag !=0 */
int *qsort_struct_perm ( void *a, int siz, int unit, int flag );
/* a version of given the "perm" array, already allocated  */
void qsort_struct_perm_ ( void *a, int siz, int unit, int *perm, int flag );


/* radix sort for array of structures, by their integer members 
  ranging from mm to m */
/* sort array "perm" according to (int/void*) array "a".
   if perm==NULL, allocate memory and for perm */
/* return the permutation array of the result of the sorting
   in the decreasing order if flag!=0 */
int *radix_sort ( void *a, int siz, int unit,int mm,int m,int *perm,int flag);

  /* bitmasks, used for bit operations */
int BITMASK_UPPER1[32] = { 0xffffffff, 0xfffffffe, 0xfffffffc, 0xfffffff8,
                           0xfffffff0, 0xffffffe0, 0xffffffc0, 0xffffff80,
                           0xffffff00, 0xfffffe00, 0xfffffc00, 0xfffff800,
                           0xfffff000, 0xffffe000, 0xffffc000, 0xffff8000,
                           0xffff0000, 0xfffe0000, 0xfffc0000, 0xfff80000,
                           0xfff00000, 0xffe00000, 0xffc00000, 0xff800000,
                           0xff000000, 0xfe000000, 0xfc000000, 0xf8000000,
                           0xf0000000, 0xe0000000, 0xc0000000, 0x80000000 };
int BITMASK_UPPER1_[32] = { 0xfffffffe, 0xfffffffc, 0xfffffff8, 0xfffffff0,
                            0xffffffe0, 0xffffffc0, 0xffffff80, 0xffffff00,
                            0xfffffe00, 0xfffffc00, 0xfffff800, 0xfffff000,
                            0xffffe000, 0xffffc000, 0xffff8000, 0xffff0000,
                            0xfffe0000, 0xfffc0000, 0xfff80000, 0xfff00000,
                            0xffe00000, 0xffc00000, 0xff800000, 0xff000000,
                            0xfe000000, 0xfc000000, 0xf8000000, 0xf0000000,
                            0xe0000000, 0xc0000000, 0x80000000, 0x00000000 };

int BITMASK_LOWER1[32] = { 0x00000000, 0x00000001, 0x00000003, 0x00000007,
                           0x0000000f, 0x0000001f, 0x0000003f, 0x0000007f,
                           0x000000ff, 0x000001ff, 0x000003ff, 0x000007ff,
                           0x00000fff, 0x00001fff, 0x00003fff, 0x00007fff,
                           0x0000ffff, 0x0001ffff, 0x0003ffff, 0x0007ffff,
                           0x000fffff, 0x001fffff, 0x003fffff, 0x007fffff,
                           0x00ffffff, 0x01ffffff, 0x03ffffff, 0x07ffffff,
                           0x0fffffff, 0x1fffffff, 0x3fffffff, 0x7fffffff };
int BITMASK_LOWER1_[32] = { 0x00000001, 0x00000003, 0x00000007, 0x0000000f,
                            0x0000001f, 0x0000003f, 0x0000007f, 0x000000ff,
                            0x000001ff, 0x000003ff, 0x000007ff, 0x00000fff,
                            0x00001fff, 0x00003fff, 0x00007fff, 0x0000ffff,
                            0x0001ffff, 0x0003ffff, 0x0007ffff, 0x000fffff,
                            0x001fffff, 0x003fffff, 0x007fffff, 0x00ffffff,
                            0x01ffffff, 0x03ffffff, 0x07ffffff, 0x0fffffff,
                            0x1fffffff, 0x3fffffff, 0x7fffffff, 0xffffffff };

int BITMASK_1[32] = { 0x00000001, 0x00000002, 0x00000004, 0x00000008,
                      0x00000010, 0x00000020, 0x00000040, 0x00000080,
                      0x00000100, 0x00000200, 0x00000400, 0x00000800,
                      0x00001000, 0x00002000, 0x00004000, 0x00008000,
                      0x00010000, 0x00020000, 0x00040000, 0x00080000,
                      0x00100000, 0x00200000, 0x00400000, 0x00800000,
                      0x01000000, 0x02000000, 0x04000000, 0x08000000,
                      0x10000000, 0x20000000, 0x40000000, 0x80000000 };
int BITMASK_31[32] = { 0xfffffffe, 0xfffffffd, 0xfffffffb, 0xfffffff7,
                       0xffffffef, 0xffffffdf, 0xffffffbf, 0xffffff7f,
                       0xfffffeff, 0xfffffdff, 0xfffffbff, 0xfffff7ff,
                       0xffffefff, 0xffffdfff, 0xffffbfff, 0xffff7fff,
                       0xfffeffff, 0xfffdffff, 0xfffbffff, 0xfff7ffff,
                       0xffefffff, 0xffdfffff, 0xffbfffff, 0xff7fffff,
                       0xfeffffff, 0xfdffffff, 0xfbffffff, 0xf7ffffff,
                       0xefffffff, 0xdfffffff, 0xbfffffff, 0x7fffffff };

int BITMASK_16[8] = { 0x0000000f, 0x000000f0, 0x00000f00, 0x0000f000,
                      0x000f0000, 0x00f00000, 0x0f000000, 0xf0000000 };
int BITMASK_UPPER16[8] = { 0xffffffff, 0xfffffff0, 0xffffff00, 0xfffff000,
                           0xffff0000, 0xfff00000, 0xff000000, 0xf0000000 };
int BITMASK_LOWER16[8] = { 0x0000000f, 0x000000ff, 0x00000fff, 0x0000ffff,
                           0x000fffff, 0x00ffffff, 0x0fffffff, 0xffffffff };
int BITMASK_FACT16[8] = { 0x1, 0x10, 0x100, 0x1000,
                          0x10000, 0x100000, 0x1000000,0x10000000 };


#endif


/****************************************** 
/*  ==== terminology for comments  ====
  range check: to check the input parameter is valid, or in the valid range.
  If a function does not have this, its comment has "no range check"
  */
   
/*  ====  rules for the name of functions and routines  ====
  init: initialization for an object, structure, etc. memory is allocated 
   if needed.
  end: compared to init, termination of structures, etc.
   free allocated memory if it exists, but not free itself.
   different from ordinary new, create, del, free.

  cpy: copy an object without allocating memory
  dup: make a duplication of an object with allocating new memory

  new: new. allocate memory for new object. also used for re-allocation from 
   the list of deleted objects
  del: delete. free memory, or insert it to the list of deleted objects

  ins : insert. insert an element (active, not deleted) to an object, possible
     at the given position.
  out : extract. extract an element, possibly specified, from an object.
        it will be not deleted.
  rm  : extract, and delete
  rmall: delete all (specified) elements of an object
  mk  : make. new+insB
  mv  : move. move the elements from an object to another,
         or change the position.

  update : update an object, possibly of specified position, to the exact,
           or to the current one.
  chg    : change the status of an object to the specified one.
  
  prv: point the previous element
  nxt: point the next element
  end: the maximum size (allocated size) of an array, etc.
  num: the (current) number of elements in an array, etc.
  kth: for the use of "first k object"
  tkth: for the use of "first k object from the end". to kth.
  rnd: random
  print: print structure and etc.
  find: search or find an specified element from the set of structures, etc.
*/

/*  ==== rules for the name of variables ====
  - use i or j for the counter in loops
  - use e for elements
  - use k for the specified rank
  - use f or flag for flagment
  - use m for maximum value or minimum value
  - use c for counters
*/


